@@ -121,8 +121,15 @@ FLINT_DLL void fmpz_mod_poly_clear(fmpz_mod_poly_t poly,
 FLINT_DLL void fmpz_mod_poly_realloc(fmpz_mod_poly_t poly, slong alloc,
                                                      const fmpz_mod_ctx_t ctx);
 
-FLINT_DLL void fmpz_mod_poly_fit_length(fmpz_mod_poly_t poly, slong len,
-                                                     const fmpz_mod_ctx_t ctx);
+FLINT_DLL void _fmpz_mod_poly_fit_length(fmpz_mod_poly_t poly, slong len);
+
+FMPZ_MOD_POLY_INLINE
+void fmpz_mod_poly_fit_length(fmpz_mod_poly_t poly, slong len,
+                                                      const fmpz_mod_ctx_t ctx)
+{
+    _fmpz_mod_poly_fit_length(poly, len);
+}
+
 
 /*  Normalisation and truncation *********************************************/
 
@@ -166,6 +173,10 @@ void fmpz_mod_poly_truncate(fmpz_mod_poly_t poly, slong len, const fmpz_mod_ctx_
 FLINT_DLL void fmpz_mod_poly_set_trunc(fmpz_mod_poly_t res,
                 const fmpz_mod_poly_t poly, slong n, const fmpz_mod_ctx_t ctx);
 
+FLINT_DLL int fmpz_mod_poly_is_canonical(const fmpz_mod_poly_t A,
+                                                     const fmpz_mod_ctx_t ctx);
+
+
 /*  Randomisation ************************************************************/
 
 FLINT_DLL void fmpz_mod_poly_randtest(fmpz_mod_poly_t f, flint_rand_t state,
@@ -276,7 +287,7 @@ void fmpz_mod_poly_one(fmpz_mod_poly_t poly, const fmpz_mod_ctx_t ctx)
 {
     fmpz_mod_poly_fit_length(poly, 1, ctx);
     fmpz_one(poly->coeffs + 0);
-    _fmpz_mod_poly_set_length(poly, !fmpz_is_one(fmpz_mod_ctx_modulus(ctx)));
+    _fmpz_mod_poly_set_length(poly, fmpz_is_one(fmpz_mod_ctx_modulus(ctx)) ? 0 : 1);
 }
 
 FLINT_DLL void fmpz_mod_poly_gen(fmpz_mod_poly_t poly, const fmpz_mod_ctx_t ctx);
@@ -312,6 +323,12 @@ FLINT_DLL void fmpz_mod_poly_set_fmpz_poly(fmpz_mod_poly_t f,
 FLINT_DLL void fmpz_mod_poly_get_fmpz_poly(fmpz_poly_t f,
                             const fmpz_mod_poly_t g, const fmpz_mod_ctx_t ctx);
 
+FLINT_DLL void fmpz_mod_poly_get_nmod_poly(nmod_poly_t f,
+		                                      const fmpz_mod_poly_t g);
+
+FLINT_DLL void fmpz_mod_poly_set_nmod_poly(fmpz_mod_poly_t f,
+		                                          const nmod_poly_t g);
+
 /*  Comparison ***************************************************************/
 
 FMPZ_MOD_POLY_INLINE 
@@ -699,6 +716,26 @@ void fmpz_mod_poly_rem(fmpz_mod_poly_t R, const fmpz_mod_poly_t A,
     fmpz_mod_poly_clear(Q, ctx);
 }
 
+FMPZ_MOD_POLY_INLINE
+void _fmpz_mod_poly_div(fmpz *Q,
+                           const fmpz *A, slong lenA, const fmpz *B, slong lenB,
+                           const fmpz_t invB, const fmpz_t p)
+{
+    fmpz * R = (fmpz *) _fmpz_vec_init(lenA);
+    _fmpz_mod_poly_divrem(Q, R, A, lenA, B, lenB, invB, p);
+    _fmpz_vec_clear(R, lenA);
+}
+
+FMPZ_MOD_POLY_INLINE
+void fmpz_mod_poly_div(fmpz_mod_poly_t Q, const fmpz_mod_poly_t A,
+                             const fmpz_mod_poly_t B, const fmpz_mod_ctx_t ctx)
+{
+    fmpz_mod_poly_t R;
+    fmpz_mod_poly_init(R, ctx);
+    fmpz_mod_poly_divrem(Q, R, A, B, ctx);
+    fmpz_mod_poly_clear(R, ctx);
+}
+
 FMPZ_MOD_POLY_INLINE
 void fmpz_mod_poly_rem_f(fmpz_t f, fmpz_mod_poly_t R, const fmpz_mod_poly_t A,
                              const fmpz_mod_poly_t B, const fmpz_mod_ctx_t ctx)
@@ -710,6 +747,20 @@ void fmpz_mod_poly_rem_f(fmpz_t f, fmpz_mod_poly_t R, const fmpz_mod_poly_t A,
     fmpz_mod_poly_clear(Q, ctx);
 }
 
+/* Divisibility testing ******************************************************/
+
+FLINT_DLL int _fmpz_mod_poly_divides_classical(fmpz * Q,
+   const fmpz * A, slong lenA, const fmpz * B, slong lenB, fmpz_mod_ctx_t ctx);
+
+FLINT_DLL int fmpz_mod_poly_divides_classical(fmpz_mod_poly_t Q,
+         const fmpz_mod_poly_t A, const fmpz_mod_poly_t B, fmpz_mod_ctx_t ctx);
+
+FLINT_DLL int _fmpz_mod_poly_divides(fmpz * Q,
+   const fmpz * A, slong lenA, const fmpz * B, slong lenB, fmpz_mod_ctx_t ctx);
+
+FLINT_DLL int fmpz_mod_poly_divides(fmpz_mod_poly_t Q,
+         const fmpz_mod_poly_t A, const fmpz_mod_poly_t B, fmpz_mod_ctx_t ctx);
+
 /*  Power series inversion ***************************************************/
 
 FLINT_DLL void _fmpz_mod_poly_inv_series_newton(fmpz * Qinv, const fmpz * Q, slong n, 
@@ -1265,11 +1316,11 @@ int fmpz_mod_poly_print_pretty(const fmpz_mod_poly_t poly, const char * x, const
 
 /* Products *****************************************************************/
 
-FLINT_DLL void _fmpz_mod_poly_product_roots_fmpz_vec(fmpz * poly, const fmpz * xs,
-                                                     slong n, const fmpz_t mod);
+FLINT_DLL void _fmpz_mod_poly_product_roots_fmpz_vec(fmpz * poly,
+                                   const fmpz * xs, slong n, const fmpz_t mod);
 
-FLINT_DLL void fmpz_mod_poly_product_roots_fmpz_vec(fmpz_poly_t poly, const fmpz * xs,
-                                                    slong n, const fmpz_t mod);
+FLINT_DLL void fmpz_mod_poly_product_roots_fmpz_vec(fmpz_mod_poly_t poly,
+                           const fmpz * xs, slong n, const fmpz_mod_ctx_t ctx);
 
 FLINT_DLL int fmpz_mod_poly_find_distinct_nonzero_roots(fmpz * roots,
                             const fmpz_mod_poly_t P, const fmpz_mod_ctx_t ctx);
@@ -1356,12 +1407,12 @@ FLINT_DLL void fmpz_mod_poly_si_sub(fmpz_mod_poly_t res, slong c,
                          const fmpz_mod_poly_t poly, const fmpz_mod_ctx_t ctx);
 
 FLINT_DLL void fmpz_mod_poly_add_fmpz(fmpz_mod_poly_t res,
-               const fmpz_mod_poly_t poly, fmpz_t c, const fmpz_mod_ctx_t ctx);
+         const fmpz_mod_poly_t poly, const fmpz_t c, const fmpz_mod_ctx_t ctx);
 
 FLINT_DLL void fmpz_mod_poly_sub_fmpz(fmpz_mod_poly_t res,
-               const fmpz_mod_poly_t poly, fmpz_t c, const fmpz_mod_ctx_t ctx);
+         const fmpz_mod_poly_t poly, const fmpz_t c, const fmpz_mod_ctx_t ctx);
 
-FLINT_DLL void fmpz_mod_poly_fmpz_sub(fmpz_mod_poly_t res, fmpz_t c,
+FLINT_DLL void fmpz_mod_poly_fmpz_sub(fmpz_mod_poly_t res, const fmpz_t c,
                          const fmpz_mod_poly_t poly, const fmpz_mod_ctx_t ctx);
 
 #ifdef __cplusplus
